import docker
import subprocess
import os
from simple_file_checksum import get_checksum
import zipfile
import requests

from rayasdk.constants import *
from rayasdk.messages import *
from rayasdk.logger import log, log_info, log_error

raya_imgs = {}
docker_client: docker.DockerClient = None


def __check_docker_instalation():
    global raya_imgs, docker_client
    # Check docker installation
    try:
        docker_client = docker.from_env()
    except docker.errors.DockerException:
        log_error(MSG_DOCKER_NOT_INSTALLER)
        # maybe required for MAC as well:
        if platform.system() == 'Windows':
            log_error(MSG_DOCKER_NOT_RUNNING)
        return False
    return True


def __remove_old_images(latest_version: str):
    log_info(('Removing old Ra-Ya images ...'))
    for image in docker_client.images.list(all=True):
        if image.tags and RAYAENV_DOCKER_IMAGE_NAME in ":".join(image.tags):
            if f'{RAYAENV_DOCKER_IMAGE_NAME}:{latest_version}' not in image.tags:
                docker_client.images.remove(image.id, force=True)
                log_info((f'\'{image.tags[0]}\' Image removed'))


def __download_image(version: str):
    log_info(f'Downloading Ra-Ya OS v{version} (only once)...')
    image_name = f'{RAYAENV_DOCKER_IMAGE_NAME}:{version}'
    try:
        if os.system(f'docker pull {image_name}') != 0:
            raise KeyboardInterrupt('')
    except KeyboardInterrupt:
        log_error('Failed to download Ra-Ya OS image. Please try again.')
        return False
    return True


def download_raya_image(version: str):
    if not __check_docker_instalation():
        return False
    __remove_old_images(version)
    if not __download_image(version=version):
        return False
    return True


def launch_command(output_file='', silent=False):
    if not os.path.exists(RAYAENV_DOCKER_UR_ROOT) or len(
            os.listdir(RAYAENV_DOCKER_UR_ROOT)) == 0:
        log_error(f'\'{RAYAENV_DOCKER_UR_ROOT}\' files does not exist')
        return False

    command = (f'cd {RAYAENV_DOCKER_UR_ROOT}/scripts && '
               f'./run.container.sh {RAYAENV_DOCKER_VERSION}')
    docker_command = f'{command} '
    if silent:
        if output_file:
            docker_command += f'> {str(output_file)}'
        else:
            docker_command += f'> /dev/null'
    else:
        if output_file:
            docker_command += f'| tee {str(output_file)}'
    return subprocess.Popen(docker_command, shell=True)


def launch_app_on_sim(app_id, args, debug=False):
    check_running_container()
    execution_route = f'/opt/raya_os/rayadevel/apps/{app_id}/'
    if debug:
        cmd = (f'python3 -m debugpy --wait-for-client --listen 5678 '
               '__main__.py ' + ' '.join(args))
    else:
        cmd = f'python3 __main__.py ' + ' '.join(args)
    command = ('cd /opt/raya_os/rayadevel && '
               'source .bashrc && '
               f'cd {execution_route} && '
               f'{cmd}')

    docker_command = (
        f'docker exec -it -u rayadevel {RAYAENV_DOCKER_IMGNAME} '
        f'bash -c "{command}"')
    ret = subprocess.call(docker_command, shell=True)


def kill_apps():
    check_running_container()
    log_info(f'Killing all running apps...')
    command = 'pkill -9 -u rayadevel'
    docker_command = (
        f'docker exec -it {RAYAENV_DOCKER_IMGNAME} {command}')
    ret = subprocess.call(docker_command, shell=True)


def check_running_container():
    # Check docker installation
    if not __check_docker_instalation():
        return False

    # Check running container
    container_running = False
    for container in docker_client.containers.list(all=True):
        if str(container.name).startswith(f'{RAYAENV_DOCKER_IMGNAME}'):
            container_running = True

    if not container_running:
        raise Exception('The container is not running')
